@@ -56,7 +56,7 @@ class AbstractAddress(models.Model):
         """
         Clean up fields
         """
-        for field in ['first_name', 'last_name', 'line1', 'line2', 'line3', 'line4', 'postcode']:
+        for field in ['first_name', 'last_name', 'line1', 'line2', 'line3', 'line4', 'state', 'postcode']:
             if self.__dict__[field]:
                self.__dict__[field] = self.__dict__[field].strip()
         
@@ -76,13 +76,13 @@ class AbstractAddress(models.Model):
         Returns a single string summary of the address,
         separating fields using commas.
         """
-        return u", ".join(self.active_address_fields())    
-        
+        return u", ".join(self.active_address_fields())
+
     def populate_alternative_model(self, address_model):
         """
         For populating an address model using the matching fields
         from this one.
-        
+
         This is used to convert a user address to a shipping address
         as part of the checkout process.
         """
@@ -90,7 +90,7 @@ class AbstractAddress(models.Model):
         for field_name in [field.name for field in self._meta.fields]:
             if field_name in destination_field_names and field_name != 'id':
                 setattr(address_model, field_name, getattr(self, field_name))
-                
+
     def active_address_fields(self):
         u"""
         Returns the non-empty components of the address, but merging the
@@ -102,17 +102,17 @@ class AbstractAddress(models.Model):
         if self.country:
             fields.append(self.country.name)
         return fields
-        
+
     def salutation(self):
         u"""Returns the salutation"""
         return u" ".join([part for part in [self.title, self.first_name, self.last_name] if part])
-        
+
     def name(self):
         """
         Returns the full name
         """
-        return u" ".join([part for part in [self.first_name, self.last_name] if part])    
-        
+        return u" ".join([part for part in [self.first_name, self.last_name] if part])
+
     def __unicode__(self):
         return self.summary
 
@@ -148,7 +148,10 @@ class AbstractShippingAddress(AbstractAddress):
     it should be read-only after that. 
     """
     phone_number = models.CharField(max_length=32, blank=True, null=True)
-    notes = models.TextField(blank=True, null=True, help_text="""Shipping notes""")
+    notes = models.TextField(blank=True, null=True,
+                             verbose_name='Courier instructions',
+                             help_text="For example, leave the parcel in the wheelie bin " \
+                                       "if I'm not in.")
     
     class Meta:
         abstract = True
@@ -178,8 +181,8 @@ class AbstractUserAddress(AbstractShippingAddress):
     user = models.ForeignKey('auth.User', related_name='addresses')
     
     # Customers can set defaults
-    is_default_for_shipping = models.BooleanField(default=False)
-    is_default_for_billing = models.BooleanField(default=False)
+    is_default_for_shipping = models.BooleanField(_("Default shipping address?"), default=False)
+    is_default_for_billing = models.BooleanField(_("Default billing address?"), default=False)
     
     # We keep track of the number of times an address has been used
     # as a shipping address so we can show the most popular ones 
@@ -192,16 +195,35 @@ class AbstractUserAddress(AbstractShippingAddress):
     date_created = models.DateTimeField(auto_now_add=True)
     
     def generate_hash(self):
-        u"""Returns a hash of the address summary."""
+        """
+        Returns a hash of the address summary
+        """
         # We use an upper-case version of the summary
         return zlib.crc32(self.summary.strip().upper().encode('UTF8'))
 
     def save(self, *args, **kwargs):
-        u"""Save a hash of the address fields"""
+        """
+        Save a hash of the address fields
+        """
         # Save a hash of the address fields so we can check whether two 
         # addresses are the same to avoid saving duplicates
         self.hash = self.generate_hash()
+        # Ensure that each user only has one default shipping address
+        # and billing address
+        self._ensure_defaults_integrity()
         super(AbstractUserAddress, self).save(*args, **kwargs)
+
+    def _ensure_defaults_integrity(self):
+        if self.is_default_for_shipping:
+            self.__class__._default_manager.filter(
+                user=self.user,
+                is_default_for_shipping=True
+            ).update(is_default_for_shipping=False)
+        if self.is_default_for_billing:
+            self.__class__._default_manager.filter(
+                user=self.user,
+                is_default_for_billing=True
+            ).update(is_default_for_billing=False)
     
     class Meta:
         abstract = True
