@@ -1,10 +1,13 @@
 from django import forms
 from django.conf import settings
 from django.db.models import get_model
+from django.forms.models import modelformset_factory, BaseModelFormSet
 from django.utils.translation import gettext_lazy as _
 
-basketline_model = get_model('basket', 'line')
-basket_model = get_model('basket', 'basket')
+from oscar.templatetags.currency_filters import currency
+
+Line = get_model('basket', 'line')
+Basket = get_model('basket', 'basket')
 Product = get_model('catalogue', 'product')
 
 
@@ -13,6 +16,11 @@ class BasketLineForm(forms.ModelForm):
 
     def clean_quantity(self):
         qty = self.cleaned_data['quantity']
+        self.check_max_allowed_quantity(qty)
+        self.check_permission(qty)
+        return qty
+
+    def check_max_allowed_quantity(self, qty):
         basket_threshold = settings.OSCAR_MAX_BASKET_QUANTITY_THRESHOLD
         if basket_threshold:
             total_basket_quantity = self.instance.basket.num_items
@@ -25,19 +33,62 @@ class BasketLineForm(forms.ModelForm):
                             'threshold': basket_threshold,
                             'basket': total_basket_quantity,
                     })
-        return qty
+
+    def check_permission(self, qty):
+        product = self.instance.product
+        is_available, reason = product.is_purchase_permitted(user=None,
+                                                             quantity=qty)
+        if not is_available:
+            raise forms.ValidationError(reason)
 
     class Meta:
-        model = basketline_model
-        exclude = ('basket', 'product', 'line_reference', )
+        model = Line
+        exclude = ('basket', 'product', 'line_reference', 'price_incl_tax')
 
 
 class SavedLineForm(forms.ModelForm):
     move_to_basket = forms.BooleanField(initial=False, required=False)
 
     class Meta:
-        model = basketline_model
-        exclude = ('basket', 'product', 'line_reference', 'quantity', )
+        model = Line
+        exclude = ('basket', 'product', 'line_reference', 'quantity', 'price_incl_tax')
+
+    def __init__(self, user, basket, *args, **kwargs):
+        self.user = user
+        self.basket = basket
+        super(SavedLineForm, self).__init__(*args, **kwargs)
+
+    def clean(self):
+        cleaned_data = super(SavedLineForm, self).clean()
+        try:
+            line = self.basket.lines.get(product=self.instance.product)
+        except Line.DoesNotExist:
+            desired_qty = self.instance.quantity
+        else:
+            desired_qty = self.instance.quantity + line.quantity
+
+        is_available, reason = self.instance.product.is_purchase_permitted(user=self.user,
+                                                                           quantity=desired_qty)
+        if not is_available:
+            raise forms.ValidationError(reason)
+        return cleaned_data
+
+
+class BaseSavedLineFormSet(BaseModelFormSet):
+
+    def __init__(self, user, basket, *args, **kwargs):
+        self.user = user
+        self.basket = basket
+        super(BaseSavedLineFormSet, self).__init__(*args, **kwargs)
+
+    def _construct_form(self, i, **kwargs):
+        return super(BaseSavedLineFormSet, self)._construct_form(i, user=self.user,
+                                                             basket=self.basket, **kwargs)
+
+
+SavedLineFormSet = modelformset_factory(Line, form=SavedLineForm,
+                                         formset=BaseSavedLineFormSet, extra=0,
+                                         can_delete=True)
 
 
 class BasketVoucherForm(forms.Form):
@@ -46,6 +97,9 @@ class BasketVoucherForm(forms.Form):
     def __init__(self, *args, **kwargs):
         return super(BasketVoucherForm, self).__init__(*args,**kwargs)
 
+    def clean_code(self):
+        return self.cleaned_data['code'].strip().upper()
+
 
 class ProductSelectionForm(forms.Form):
     product_id = forms.IntegerField(min_value=1)
@@ -63,9 +117,10 @@ class AddToBasketForm(forms.Form):
     product_id = forms.IntegerField(widget=forms.HiddenInput(), min_value=1)
     quantity = forms.IntegerField(initial=1, min_value=1)
 
-    def __init__(self, basket, instance, *args, **kwargs):
+    def __init__(self, basket, user, instance, *args, **kwargs):
         super(AddToBasketForm, self).__init__(*args, **kwargs)
         self.basket = basket
+        self.user = user
         self.instance = instance
         if instance:
             if instance.is_group:
@@ -73,12 +128,22 @@ class AddToBasketForm(forms.Form):
             else:
                 self._create_product_fields(instance)
 
-    def clean_product_id(self):
+    def clean(self):
         id = self.cleaned_data['product_id']
         product = Product.objects.get(id=id)
-        if not product.has_stockrecord or not product.stockrecord.is_available_to_buy:
-            raise forms.ValidationError(_("This product is not available for purchase"))
-        return id
+        qty = self.cleaned_data.get('quantity', 1)
+        try:
+            line = self.basket.lines.get(product=product)
+        except Line.DoesNotExist:
+            desired_qty = qty
+        else:
+            desired_qty = qty + line.quantity
+
+        is_available, reason = product.is_purchase_permitted(user=self.user,
+                                                             quantity=desired_qty)
+        if not is_available:
+            raise forms.ValidationError(reason)
+        return self.cleaned_data
 
     def clean_quantity(self):
         qty = self.cleaned_data['quantity']
@@ -104,10 +169,14 @@ class AddToBasketForm(forms.Form):
         choices = []
         for variant in item.variants.all():
             if variant.has_stockrecord:
-                summary = u"%s (%s) - %.2f" % (variant.get_title(), variant.attribute_summary(),
-                                               variant.stockrecord.price_incl_tax)
+                attr_summary = variant.attribute_summary()
+                if attr_summary:
+                    attr_summary = "(%s)" % attr_summary
+                summary = u"%s %s - %s" % (variant.get_title(), attr_summary,
+                                           currency(variant.stockrecord.price_incl_tax))
                 choices.append((variant.id, summary))
-        self.fields['product_id'] = forms.ChoiceField(choices=tuple(choices))
+        self.fields['product_id'] = forms.ChoiceField(choices=tuple(choices),
+                                                      label="Variant")
 
     def _create_product_fields(self, item):
         u"""Add the product option fields."""
@@ -124,3 +193,6 @@ class AddToBasketForm(forms.Form):
         self.fields[option.code] = forms.CharField()
 
 
+class SimpleAddToBasketForm(AddToBasketForm):
+    quantity = forms.IntegerField(initial=1, min_value=1, widget=forms.HiddenInput)
+
