@@ -2,17 +2,20 @@ from django.contrib import messages
 from django.core.urlresolvers import reverse
 from django.db.models import get_model
 from django.http import HttpResponseRedirect, Http404
-from django.views.generic import ListView, FormView, View
-from django.forms.models import modelformset_factory
+from django.views.generic import FormView, View
 from django.utils.translation import ugettext_lazy as _
 from django.core.exceptions import ObjectDoesNotExist
 
 from extra_views import ModelFormSetView
-from oscar.apps.basket.forms import BasketLineForm, AddToBasketForm, \
-    BasketVoucherForm, SavedLineForm, ProductSelectionForm
-from oscar.apps.basket.signals import basket_addition
-from oscar.core.loading import import_module
-import_module('offer.utils', ['Applicator'], locals())
+from oscar.apps.basket.signals import basket_addition, voucher_addition
+from oscar.core.loading import get_class, get_classes
+Applicator = get_class('offer.utils', 'Applicator')
+BasketLineForm, AddToBasketForm, BasketVoucherForm, \
+        SavedLineFormSet, SavedLineForm, ProductSelectionForm = get_classes(
+            'basket.forms', ('BasketLineForm', 'AddToBasketForm',
+                             'BasketVoucherForm', 'SavedLineFormSet',
+                             'SavedLineForm', 'ProductSelectionForm'))
+Repository = get_class('shipping.repository', ('Repository'))
 
 
 class BasketView(ModelFormSetView):
@@ -24,38 +27,79 @@ class BasketView(ModelFormSetView):
     template_name='basket/basket.html'
 
     def get_queryset(self):
-        return self.request.basket.lines.all()
+        return self.request.basket.all_lines()
+
+    def get_default_shipping_method(self, basket):
+        return Repository().get_default_shipping_method(self.request.user, self.request.basket)
+
+    def get_basket_warnings(self, basket):
+        """
+        Return a list of warnings that apply to this basket
+        """
+        warnings = []
+        for line in basket.all_lines():
+            warning = line.get_warning()
+            if warning:
+                warnings.append(warning)
+        return warnings
+
+    def get_upsell_messages(self, basket):
+        offers = Applicator().get_offers(self.request, basket)
+        messages = []
+        for offer in offers:
+            if offer.is_condition_partially_satisfied(basket):
+                data = {
+                    'message': offer.get_upsell_message(basket),
+                    'offer': offer
+                }
+                messages.append(data)
+        return messages
 
     def get_context_data(self, **kwargs):
         context = super(BasketView, self).get_context_data(**kwargs)
         context['voucher_form'] = BasketVoucherForm()
+        method = self.get_default_shipping_method(self.request.basket)
+        context['shipping_method'] = method
+        context['shipping_charge_incl_tax'] = method.basket_charge_incl_tax()
+        context['order_total_incl_tax'] = self.request.basket.total_incl_tax + method.basket_charge_incl_tax()
+        context['basket_warnings'] = self.get_basket_warnings(self.request.basket)
+        context['upsell_messages'] = self.get_upsell_messages(self.request.basket)
 
         if self.request.user.is_authenticated():
             try:
                 saved_basket = self.basket_model.saved.get(owner=self.request.user)
-                saved_queryset = saved_basket.lines.all().select_related('product', 'product__stockrecord')
-                SavedFormset = modelformset_factory(self.model, form=SavedLineForm, extra=0, can_delete=True)
-                formset = SavedFormset(queryset=saved_queryset)
-                context['saved_formset'] = formset
             except self.basket_model.DoesNotExist:
                 pass
+            else:
+                if not saved_basket.is_empty:
+                    saved_queryset = saved_basket.all_lines().select_related('product', 'product__stockrecord')
+                    formset = SavedLineFormSet(user=self.request.user,
+                                               basket=self.request.basket,
+                                               queryset=saved_queryset,
+                                               prefix='saved')
+                    context['saved_formset'] = formset
         return context
 
     def get_success_url(self):
+        messages.success(self.request, _("Basket updated"))
         return self.request.META.get('HTTP_REFERER', reverse('basket:summary'))
 
     def formset_valid(self, formset):
-        needs_auth = False
+        save_for_later = False
         for form in formset:
             if hasattr(form, 'cleaned_data') and form.cleaned_data['save_for_later']:
                 line = form.instance
                 if self.request.user.is_authenticated():
                     self.move_line_to_saved_basket(line)
                     messages.info(self.request, _(u"'%(title)s' has been saved for later" % {'title': line.product}))
+                    save_for_later = True
                 else:
-                    needs_auth = True
-        if needs_auth:
-            messages.error(self.request, "You can't save an item for later if you're not logged in!")
+                    messages.error(self.request, "You can't save an item for later if you're not logged in!")
+                    return HttpResponseRedirect(self.get_success_url())
+
+        if save_for_later:
+            # No need to call super if we're moving lines to the saved basket
+            return HttpResponseRedirect(self.get_success_url())
         return super(BasketView, self).formset_valid(formset)
 
     def move_line_to_saved_basket(self, line):
@@ -63,7 +107,11 @@ class BasketView(ModelFormSetView):
         saved_basket.merge_line(line)
 
     def formset_invalid(self, formset):
-        messages.info(self.request, _("There was a problem updating your basket, please check that all quantities are numbers"))
+        errors = []
+        for error_dict in formset.errors:
+            errors += [error_list.as_text() for error_list in error_dict.values()]
+        msg = "Your basket couldn't be updated because:\n%s" % "\n".join(errors)
+        messages.warning(self.request, msg)
         return super(BasketView, self).formset_invalid(formset)
 
 
@@ -88,6 +136,7 @@ class BasketAddView(FormView):
             kwargs['instance'] = product_select_form.cleaned_data['product_id']
         else:
             raise Http404()
+        kwargs['user'] = self.request.user
         kwargs['basket'] = self.request.basket
         return kwargs
 
@@ -120,6 +169,7 @@ class BasketAddView(FormView):
 class VoucherAddView(FormView):
     form_class = BasketVoucherForm
     voucher_model = get_model('voucher', 'voucher')
+    add_signal = voucher_addition
 
     def get(self, request, *args, **kwargs):
         return HttpResponseRedirect(reverse('basket:summary'))
@@ -136,6 +186,11 @@ class VoucherAddView(FormView):
 
         self.request.basket.vouchers.add(voucher)
 
+        # Raise signal
+        self.add_signal.send(sender=self, 
+                             basket=self.request.basket, 
+                             voucher=voucher)
+
         # Recalculate discounts to see if the voucher gives any
         discounts_before = self.request.basket.get_discounts()
         self.request.basket.remove_discounts()
@@ -170,7 +225,8 @@ class VoucherAddView(FormView):
         return HttpResponseRedirect(self.request.META.get('HTTP_REFERER', reverse('basket:summary')))
 
     def form_invalid(self, form):
-        return HttpResponseRedirect(reverse('basket:summary'))
+        messages.error(self.request, _("Please enter a voucher code"))
+        return HttpResponseRedirect(reverse('basket:summary')+'#voucher')
 
 
 class VoucherRemoveView(View):
@@ -199,6 +255,7 @@ class VoucherRemoveView(View):
 class SavedView(ModelFormSetView):
     model = get_model('basket', 'line')
     basket_model = get_model('basket', 'basket')
+    formset_class = SavedLineFormSet
     form_class = SavedLineForm
     extra = 0
     can_delete = True
@@ -209,36 +266,34 @@ class SavedView(ModelFormSetView):
     def get_queryset(self):
         try:
             saved_basket = self.basket_model.saved.get(owner=self.request.user)
-            return saved_basket.lines.all().select_related('product', 'product__stockrecord')
+            return saved_basket.all_lines().select_related('product', 'product__stockrecord')
         except self.basket_model.DoesNotExist:
             return []
 
     def get_success_url(self):
         return self.request.META.get('HTTP_REFERER', reverse('basket:summary'))
 
+    def get_formset_kwargs(self):
+        kwargs = super(SavedView, self).get_formset_kwargs()
+        kwargs['prefix'] = 'saved'
+        kwargs['basket'] = self.request.basket
+        kwargs['user'] = self.request.user
+        return kwargs
+
     def formset_valid(self, formset):
+        is_move = False
         for form in formset:
             if form.cleaned_data['move_to_basket']:
+                is_move = True
                 msg = "'%s' has been moved back to your basket" % form.instance.product
                 messages.info(self.request, msg)
                 real_basket = self.request.basket
                 real_basket.merge_line(form.instance)
+        if is_move:
+            return HttpResponseRedirect(self.get_success_url())
+
         return super(SavedView, self).formset_valid(formset)
 
     def formset_invalid(self, formset):
+        messages.error(self.request, '\n'.join(error for ed in formset.errors for el in ed.values() for error in el))
         return HttpResponseRedirect(self.request.META.get('HTTP_REFERER', reverse('basket:summary')))
-
-
-
-#    def do_bulk_load(self, basket):
-#        num_additions = 0
-#        num_not_found = 0
-#        for sku in re.findall(r"[\d -]{5,}", self.request.POST['source_text']):
-#            try:
-#                item = Product.objects.get(upc=sku)
-#                basket.add_product(item)
-#                num_additions += 1
-#            except Product.DoesNotExist:
-#                num_not_found += 1
-#        messages.info(self.request, "Added %d items to your basket (%d missing)" % (num_additions, num_not_found))
-#
\ No newline at end of file
