@@ -1,23 +1,27 @@
 from itertools import chain
 from decimal import Decimal as D
+import hashlib
+import datetime
 
 from django.db import models
 from django.contrib.auth.models import User
 from django.template.defaultfilters import slugify
 from django.utils.translation import ugettext_lazy as _
 from django.db.models import Sum
-from django.core.exceptions import ObjectDoesNotExist
+from django.conf import settings
+
+from oscar.apps.order.exceptions import (InvalidOrderStatus, InvalidLineStatus,
+                                         InvalidShippingEvent)
 
 
 class AbstractOrder(models.Model):
     """
     The main order model
     """
-    
     number = models.CharField(_("Order number"), max_length=128, db_index=True)
     # We track the site that each order is placed within
     site = models.ForeignKey('sites.Site')
-    basket = models.ForeignKey('basket.Basket', null=True, blank=True)
+    basket_id = models.PositiveIntegerField(null=True, blank=True)
     # Orders can be anonymous so we don't always have a customer ID
     user = models.ForeignKey(User, related_name='orders', null=True, blank=True)
     # Billing address is not always required (eg paying by gift card)
@@ -40,9 +44,39 @@ class AbstractOrder(models.Model):
     
     # Use this field to indicate that an order is on hold / awaiting payment
     status = models.CharField(_("Status"), max_length=100, null=True, blank=True)
+
+    guest_email = models.EmailField(_("Guest email address"), null=True, blank=True)
     
     # Index added to this field for reporting
     date_placed = models.DateTimeField(auto_now_add=True, db_index=True)
+
+    # Dict of available status changes
+    pipeline = getattr(settings,  'OSCAR_ORDER_STATUS_PIPELINE', {})
+    cascade = getattr(settings,  'OSCAR_ORDER_STATUS_CASCADE', {})
+
+    @classmethod
+    def all_statuses(cls):
+        return cls.pipeline.keys()
+
+    def available_statuses(self):
+        return self.pipeline.get(self.status, ())
+
+    def set_status(self, new_status):
+        if new_status == self.status:
+            return
+        if new_status not in self.available_statuses():
+            raise InvalidOrderStatus("'%s' is not a valid status for order %s (currency status: '%s')" %
+                                     (new_status, self.number, self.status))
+        self.status = new_status
+        if new_status in self.cascade:
+            for line in self.lines.all():
+                line.status = self.cascade[self.status]
+                line.save()
+        self.save()
+    
+    @property
+    def is_anonymous(self):
+        return self.user is None
     
     @property
     def basket_total_incl_tax(self):
@@ -101,7 +135,7 @@ class AbstractOrder(models.Model):
     
     @property
     def num_items(self):
-        u"""
+        """
         Returns the number of items in this order.
         """
         num_items = 0
@@ -153,6 +187,15 @@ class AbstractOrder(models.Model):
     def __unicode__(self):
         return u"#%s" % (self.number,)
 
+    def verification_hash(self):
+        return hashlib.md5('%s%s' % (self.number, settings.SECRET_KEY)).hexdigest()
+
+    @property
+    def email(self):
+        if not self.user:
+            return self.guest_email
+        return self.user.email
+
 
 class AbstractOrderNote(models.Model):
     """
@@ -168,11 +211,15 @@ class AbstractOrderNote(models.Model):
     user = models.ForeignKey('auth.User', null=True)
     
     # We allow notes to be classified although this isn't always needed
-    INFO, WARNING, ERROR = 'Info', 'Warning', 'Error'
+    INFO, WARNING, ERROR, SYSTEM = 'Info', 'Warning', 'Error', 'System'
     note_type = models.CharField(max_length=128, null=True)
     
     message = models.TextField()
-    date = models.DateTimeField(auto_now_add=True)
+    date_created = models.DateTimeField(auto_now_add=True)
+    date_updated = models.DateTimeField(auto_now=True)
+
+    # Notes can only be edited for 5 minutes after being created
+    editable_lifetime = 300
     
     class Meta:
         abstract = True
@@ -180,6 +227,11 @@ class AbstractOrderNote(models.Model):
     def __unicode__(self):
         return u"'%s' (%s)" % (self.message[0:50], self.user)
 
+    def is_editable(self):
+        if self.note_type == self.SYSTEM:
+            return False
+        return (datetime.datetime.now() - self.date_updated).seconds < self.editable_lifetime
+
 
 class AbstractCommunicationEvent(models.Model):
     """
@@ -187,7 +239,7 @@ class AbstractCommunicationEvent(models.Model):
     as an confirmation email being sent.
     """
     order = models.ForeignKey('order.Order', related_name="communication_events")
-    type = models.ForeignKey('customer.CommunicationEventType')
+    event_type = models.ForeignKey('customer.CommunicationEventType')
     date = models.DateTimeField(auto_now_add=True)
     
     class Meta:
@@ -212,7 +264,9 @@ class AbstractLine(models.Model):
     partner = models.ForeignKey('partner.Partner', related_name='order_lines', blank=True, null=True, on_delete=models.SET_NULL)
     partner_name = models.CharField(_("Partner name"), max_length=128)
     partner_sku = models.CharField(_("Partner SKU"), max_length=128)
+
     title = models.CharField(_("Title"), max_length=255)
+    upc = models.CharField(_("UPC"), max_length=128, blank=True, null=True)
     
     # We don't want any hard links between orders and the products table so we allow
     # this link to be NULLable.
@@ -248,6 +302,31 @@ class AbstractLine(models.Model):
     
     # Estimated dispatch date - should be set at order time
     est_dispatch_date = models.DateField(blank=True, null=True)
+
+    pipeline = getattr(settings,  'OSCAR_LINE_STATUS_PIPELINE', {})
+
+    @classmethod
+    def all_statuses(cls):
+        return cls.pipeline.keys()
+
+    def available_statuses(self):
+        return self.pipeline.get(self.status, ())
+
+    def set_status(self, new_status):
+        if new_status == self.status:
+            return
+        if new_status not in self.available_statuses():
+            raise InvalidLineStatus("'%s' is not a valid status (current status: '%s')" % (
+                new_status, self.status))
+        self.status = new_status
+        self.save()
+
+    @property
+    def category(self):
+        """
+        Used by Google analytics tracking
+        """
+        return None
     
     @property
     def description(self):
@@ -270,17 +349,25 @@ class AbstractLine(models.Model):
     @property
     def discount_excl_tax(self):
         return self.line_price_before_discounts_excl_tax - self.line_price_excl_tax
+
+    @property
+    def line_price_tax(self):
+        return self.line_price_incl_tax - self.line_price_excl_tax
+
+    @property
+    def unit_price_tax(self):
+        return self.unit_price_incl_tax - self.unit_price_excl_tax
     
     @property
     def shipping_status(self):
-        u"""Returns a string summary of the shipping status of this line"""
-        status_map = self._shipping_event_history()
+        """Returns a string summary of the shipping status of this line"""
+        status_map = self.shipping_event_breakdown()
         if not status_map:
             return ''
         
         events = []    
         last_complete_event_name = None
-        for event_dict in status_map:
+        for event_dict in status_map.values():
             if event_dict['quantity'] == self.quantity:
                 events.append(event_dict['name'])
                 last_complete_event_name = event_dict['name']    
@@ -288,34 +375,42 @@ class AbstractLine(models.Model):
                 events.append("%s (%d/%d items)" % (event_dict['name'], 
                                                     event_dict['quantity'], self.quantity))
         
-        if last_complete_event_name == status_map[-1]['name']:
+        if last_complete_event_name == status_map.values()[-1]['name']:
             return last_complete_event_name
             
         return ', '.join(events)
     
-    def has_shipping_event_occurred(self, event_type):
-        u"""Checks whether this line has passed a given shipping event"""
-        for event_dict in self._shipping_event_history():
-            if event_dict['name'] == event_type.name and event_dict['quantity'] == self.quantity:
+    def has_shipping_event_occurred(self, event_type, quantity=None):
+        """
+        Check whether this line has passed a given shipping event
+        """
+        if not quantity:
+            quantity = self.quantity
+        for name, event_dict in self.shipping_event_breakdown().items():
+            if name == event_type.name and event_dict['quantity'] == self.quantity:
                 return True
         return False
     
     @property
     def is_product_deleted(self):
         return self.product == None
-    
-    def _shipping_event_history(self):
-        u"""
-        Returns a list of shipping events"""
+
+    def shipping_event_breakdown(self):
+        """
+        Returns a dict of shipping events that this line has been through
+        """
         status_map = {}
         for event in self.shippingevent_set.all():
-            event_name = event.event_type.name
+            event_type = event.event_type
+            event_name = event_type.name
             event_quantity = event.line_quantities.get(line=self).quantity
             if event_name in status_map:
                 status_map[event_name]['quantity'] += event_quantity
             else:
-                status_map[event_name] = {'name': event_name, 'quantity': event_quantity}
-        return list(status_map.values())
+                status_map[event_name] = {'name': event_name,
+                                          'event_type': event.event_type,
+                                          'quantity': event_quantity}
+        return status_map
     
     class Meta:
         abstract = True
@@ -373,8 +468,8 @@ class AbstractPaymentEventType(models.Model):
     """
     Payment events are things like 'Paid', 'Failed', 'Refunded'
     """
-    name = models.CharField(max_length=128)
-    code = models.SlugField(max_length=128)
+    name = models.CharField(max_length=128, unique=True)
+    code = models.SlugField(max_length=128, unique=True)
     sequence_number = models.PositiveIntegerField(default=0)
     
     def save(self, *args, **kwargs):
@@ -397,6 +492,7 @@ class AbstractPaymentEvent(models.Model):
     payment being taken for 2 items, or 1 item being dispatched.
     """
     order = models.ForeignKey('order.Order', related_name='payment_events')
+    amount = models.DecimalField(decimal_places=2, max_digits=12)
     lines = models.ManyToManyField('order.Line', through='PaymentEventQuantity')
     event_type = models.ForeignKey('order.PaymentEventType')
     date = models.DateTimeField(auto_now_add=True)
@@ -406,7 +502,7 @@ class AbstractPaymentEvent(models.Model):
         verbose_name_plural = _("Payment events")
         
     def __unicode__(self):
-        return u"Payment event for order #%s" % self.order
+        return u"Payment event for order %s" % self.order
 
 
 class PaymentEventQuantity(models.Model):
@@ -418,8 +514,11 @@ class PaymentEventQuantity(models.Model):
     quantity = models.PositiveIntegerField()
 
 
+# SHIPPING EVENTS
+
+
 class AbstractShippingEvent(models.Model):    
-    u"""
+    """
     An event is something which happens to a group of lines such as
     1 item being dispatched.
     """
@@ -444,19 +543,26 @@ class AbstractShippingEvent(models.Model):
 
 
 class ShippingEventQuantity(models.Model):
-    u"""A "through" model linking lines to shipping events"""
+    """
+    A "through" model linking lines to shipping events
+    """
     event = models.ForeignKey('order.ShippingEvent', related_name='line_quantities')
     line = models.ForeignKey('order.Line')
     quantity = models.PositiveIntegerField()
 
     def _check_previous_events_are_complete(self):
-        u"""Checks whether previous shipping events have passed"""
-        previous_events = ShippingEventQuantity._default_manager.filter(line=self.line, 
-                                                                        event__event_type__sequence_number__lt=self.event.event_type.sequence_number)
-        self.quantity = int(self.quantity)
-        for event_quantities in previous_events:
-            if event_quantities.quantity < self.quantity:
-                raise ValueError("Invalid quantity (%d) for event type (a previous event has not been fully passed)" % self.quantity)
+        """
+        Checks whether previous shipping events have passed
+        """
+        # Quantity of the proposd event must have occurred for
+        # the previous events in the sequence.
+        previous_event_types = self.event.event_type.get_prerequisites()
+        for event_type in previous_event_types:
+            quantity = ShippingEventQuantity._default_manager.filter(
+                line=self.line,
+                event__event_type=event_type).aggregate(Sum('quantity'))['quantity__sum']
+            if quantity is None or quantity < int(self.quantity):
+                raise InvalidShippingEvent("This shipping event is not permitted")
 
     def _check_new_quantity(self):
         quantity_row = ShippingEventQuantity._default_manager.filter(line=self.line, 
@@ -471,6 +577,7 @@ class ShippingEventQuantity(models.Model):
         # Default quantity to full quantity of line
         if not self.quantity:
             self.quantity = self.line.quantity
+        self.quantity = int(self.quantity)
         self._check_previous_events_are_complete()
         self._check_new_quantity()
         super(ShippingEventQuantity, self).save(*args, **kwargs)
@@ -480,13 +587,13 @@ class ShippingEventQuantity(models.Model):
 
 
 class AbstractShippingEventType(models.Model):
-    u"""Shipping events are things like 'OrderPlaced', 'Acknowledged', 'Dispatched', 'Refunded'"""
-    # Code is used in forms
-    code = models.CharField(max_length=128)
+    """
+    Shipping events are things like 'OrderPlaced', 'Acknowledged', 'Dispatched', 'Refunded'
+    """
     # Name is the friendly description of an event
-    name = models.CharField(max_length=255)
+    name = models.CharField(max_length=255, unique=True)
     # Code is used in forms
-    code = models.SlugField(max_length=128)
+    code = models.SlugField(max_length=128, unique=True)
     is_required = models.BooleanField(default=True, help_text="This event must be passed before the next shipping event can take place")
     # The normal order in which these shipping events take place
     sequence_number = models.PositiveIntegerField(default=0)
@@ -503,6 +610,11 @@ class AbstractShippingEventType(models.Model):
         
     def __unicode__(self):
         return self.name
+
+    def get_prerequisites(self):
+        return self.__class__._default_manager.filter(
+            is_required=True,
+            sequence_number__lt=self.sequence_number).order_by('sequence_number')
         
         
 class AbstractOrderDiscount(models.Model):
@@ -513,8 +625,8 @@ class AbstractOrderDiscount(models.Model):
     separately even though in reality, the discounts are applied at the line level.
     """
     order = models.ForeignKey('order.Order', related_name="discounts")
-    offer = models.ForeignKey('offer.ConditionalOffer', null=True, on_delete=models.SET_NULL)
-    voucher = models.ForeignKey('voucher.Voucher', related_name="discount_vouchers", null=True, on_delete=models.SET_NULL)
+    offer_id = models.PositiveIntegerField(blank=True, null=True)
+    voucher_id = models.PositiveIntegerField(blank=True, null=True)
     voucher_code = models.CharField(_("Code"), max_length=128, db_index=True, null=True)
     amount = models.DecimalField(decimal_places=2, max_digits=12, default=0)
     
@@ -523,6 +635,22 @@ class AbstractOrderDiscount(models.Model):
         
     def __unicode__(self):
         return u"Discount of %r from order %s" % (self.amount, self.order)    
+
+    @property
+    def offer(self):
+        Offer = models.get_model('offer', 'ConditionalOffer')
+        try:
+            return Offer.objects.get(id=self.offer_id)
+        except Offer.DoesNotExist:
+            return None
+
+    @property
+    def voucher(self):
+        Voucher = models.get_model('voucher', 'Voucher')
+        try:
+            return Voucher.objects.get(id=self.offer_id)
+        except Voucher.DoesNotExist:
+            return None
         
     def description(self):
         if self.voucher_code:
